/*
 * Copyright 2002-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.didion.vfs.provider.crypt;

import org.apache.commons.vfs.provider.LayeredFileName;
import org.apache.commons.vfs.provider.compressed.CompressedFileFileProvider;
import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemOptions;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.Capability;
import org.apache.commons.vfs.FileSystemConfigBuilder;
import org.apache.commons.vfs.FileSystem;
import org.apache.commons.vfs.FileName;

import java.util.Collection;
import java.util.Arrays;

public class CryptFileProvider extends CompressedFileFileProvider {
    protected final static Collection capabilities = Arrays.asList(new Capability[] {
        Capability.GET_LAST_MODIFIED,
        Capability.GET_TYPE,
        Capability.LIST_CHILDREN,
        Capability.WRITE_CONTENT,
        Capability.READ_CONTENT,
        Capability.URI,
        Capability.COMPRESS
    });

    public Collection getCapabilities() {
        return capabilities;
    }

    protected FileSystem createFileSystem(
            final FileName name,
            final FileObject file,
            final FileSystemOptions fileSystemOptions)
            throws FileSystemException {

        FileName outer = ((LayeredFileName) name).getOuterName();

        CipherInfo cipherInfo = null;
        FileName root = outer;
        boolean associate = false;
        while (null == cipherInfo && root != null) {
            cipherInfo = CryptFileSystemConfigBuilder.getInstance().
                    getCipherInfo(fileSystemOptions, root.getURI());
            if (null == cipherInfo) {
                associate = true;
                root = root.getParent();
            }
        }
        if (null == cipherInfo) {
            associate = true;
            cipherInfo = CryptFileSystemConfigBuilder.getInstance().
                    getDefaultCipherInfo(fileSystemOptions);
            if (null == cipherInfo) {
                cipherInfo = new CipherInfo();
            }
        }

        if (associate) {
            CryptFileSystemConfigBuilder.getInstance().setCipherInfo(
                    fileSystemOptions, outer.getURI(), cipherInfo);
        }

        return new CryptFileSystem(name, file, fileSystemOptions, cipherInfo);
    }

    public FileSystemConfigBuilder getConfigBuilder() {
        return CryptFileSystemConfigBuilder.getInstance();
    }
}